local game = require 'game'
local sprite = game.sprite
local social = game.socialHelper
local self = sprite:create(game.gm.getLocalIndicator(0, "self"),'game_master')
local analytics = game.analyticsHelper
sprite:clearcache()


EVENT_YAHOO 				= 10
EVENT_YAHOO_REPLAY 			= 11
EVENT_YAHOO_STOP 			= 12
EVENT_CREATE_START_BUTTON 	= 13
EVENT_MISSLE_POWERUP		= 14
EVENT_SWITCH_KEYS_POWERUP	= 15
EVENT_CREATE_MINE           = 16 -- Args: x, y
EVENT_FINISHED_RACE         = 17 -- Args: playerId, finishTime
EVENT_HIT_MSG               = 18
EVENT_TALK_MSG              = 19 -- Args: msg_text, sender_id
EVENT_HERO_FINISH_NAME      = 20 -- Args: heroId, heroName
------------------------------------------------------------------------------------
g_players_positions = {} -- used for power-ups
print("g_players inited")

yahoo_replies = {}
local i

for i=1, multi_players_total do
	yahoo_replies[i] = 0
end
---------------------------------------------------------
function count_yahoo_replies()
	local sum = 0
	for i=1, #yahoo_replies do
		sum = yahoo_replies[i] + sum
	end
	return sum
end	
-----------------------------------------------------------------------------------
-- Start button creation sync events binding
-----------------------------------------------------------------------------------
game.networkingEvents.bind(game.enums.EngineEvents.GAME_ABOUT_TO_START, function(args)    
    self.create_loading_indication({})
end)

game.networkingEvents.bind(game.enums.EngineEvents.GAME_STARTED, function(args)
    -- TODO: Enable "GO"
    self.create_start_button({})
end)

function g_CheckYahooReplies()    
    print("=================g_CheckYahooReplies started =================")
    local sum_  = count_yahoo_replies()
    local playersInRoom = game.networking.getNumberOfPlayersInRoom()
    print("sum_ = " .. sum_ .. " playersInRoom = " .. playersInRoom)

    if (sum_ == playersInRoom - 1) then
        print("start game")
        game.networkingEvents.send(EVENT_CREATE_START_BUTTON, {a=1})                        
        game.networkingEvents.unbind(EVENT_YAHOO_REPLAY)            
        game.networking.startGame()
        return true
    else
        return false
    end            
    print("=================g_CheckYahooReplies ended =================")
end


if(game.networking.getNumberOfPlayersInRoom() > 1) then 
    -- yahoo replies will be checked by timer in host       
    self.start_yahoo_timers()
	game.networkingEvents.bind(EVENT_YAHOO_REPLAY, function(args)
		local my_id = tonumber(args.id)
		yahoo_replies[my_id] = 1        
    end)
else
    -- Lonley player - play with bots!
    print("self.call_start_game() called!!!")
    game.networking.startGame()    
    
end

if(IS_HOST == 0) then
	game.networkingEvents.bind(EVENT_YAHOO, function(args)
		game.networkingEvents.send(EVENT_YAHOO_REPLAY, {id=multi_player_id_me})
	end)

	game.networkingEvents.bind(EVENT_CREATE_START_BUTTON, function(args)								
		game.networkingEvents.unbind(EVENT_CREATE_START_BUTTON)
		game.networkingEvents.unbind(EVENT_YAHOO)
	end)
end

print("I'am host = " .. IS_HOST.. ", game.networking.getNumberOfPlayersInRoom() = " .. game.networking.getNumberOfPlayersInRoom())
-----------------------------------------------------------------------------------
-- OOPS events
-----------------------------------------------------------------------------------
game.networkingEvents.bind(game.enums.EngineEvents.PLAYER_LEFT_ROOM, function (args)
		-- check if the hero is after finish line
		local playerNumber = tonumber(args.playerNumber)
        print("Player ID: " .. playerNumber .. " left room or was disconnected")
		local afterFinishLine = false
		for i=1, hero.Length do
			if hero[i].my_serial_id == playerNumber then
				if hero[i].animation == A_LOSE or hero[i].animation == A_WIN_JUMP then 
					afterFinishLine = true
				end
				break
			end
		end
		if afterFinishLine == false then 
			game.networking.destroyPlayerObjects(playerNumber)
			self.on_player_left_listener({player_id = playerNumber})
		end
		
	end)
-----------------------------------------------------------------------------------
game.networkingEvents.bind(game.enums.EngineEvents.DISCONNECTED, function (args)		
		analytics.endLevel()		
		self.on_my_disconnect({})
	end)

-----------------------------------------------------------------------------------
-- Finish line names events
-----------------------------------------------------------------------------------
game.networkingEvents.bind(EVENT_HERO_FINISH_NAME, function(args)
        -- Args: heroId, heroName
        -- filter hero_name of projections
        
        local heroId = tonumber(args["heroId"])
        local heroName = args["heroName"]
        
        print("EVENT_HERO_FINISH_NAME:: ---id = " .. heroId .. ", heroName = " .. heroName)

        local projection = filter(spriteList(hero_name), function(heroNameSprite)
            print("EVENT_HERO_FINISH_NAME:: tested sprite id = " .. heroNameSprite.my_id .. ", multi_player_id_me = " .. multi_player_id_me)
            return (heroNameSprite.my_id == heroId and 
                   heroNameSprite.my_id ~= multi_player_id_me and 
                   heroNameSprite.is_bot == 0)
        end)

        if #projection > 0 then
           print(heroName .."  " .. #projection .. " projections found") 
           projection[1].canvas.name.text = heroName
        else
            print("This should never happen," .. heroName .. " projection not found")
        end

    end)
-----------------------------------------------------------------------------------
-- Mine events
-----------------------------------------------------------------------------------
game.networkingEvents.bind(EVENT_CREATE_MINE, function(args)
        self.on_create_mine { x=tonumber(args.x), y=tonumber(args.y), z=0, owner_id = tonumber(args.owner_id) }
    end)
-----------------------------------------------------------------------------------
-- Multiplayer end game logic
-----------------------------------------------------------------------------------

local finishTimes = {} -- List of {playerId, finishTime}

local function findHeroById(id)
    local result

    forEachSprite(hero, function(currentHero)
        if currentHero.my_id == id then
            result = hero
        end
    end)

    return result
end

local function traceFinishTimes()
    for place, playerIdAndFinishTime in ipairs(finishTimes) do
        print("WinCondition: At #"..place..": "..
              playerIdAndFinishTime.playerId.." finished at "..
              playerIdAndFinishTime.finishTime)
    end
end

local function sortFinishTimesAscending()
    -- Sort by finishTime first, then by playerId
    -- In the very rare case that two players finish at the same time, the one
    -- with lower player ID wins.
    table.sort(finishTimes, function(a, b)
        if a.finishTime ~= b.finishTime then
            return a.finishTime < b.finishTime
        else
            return a.playerId < b.playerId
        end
    end)
end

local function getPlayersWhoDidntReportTime()
    local playersWhoReported = Set(map(finishTimes, function(record) return record.playerId end))
    local allPlayers = Set { 1, 2, 3, 4 }

    return allPlayers - playersWhoReported
end

local function reportPlayersPositions()

    if finish_x <= 0 then
		-- Finish line is unknown, don't report places
		return
	end
    print("reportPlayersPositions():: --------------started------------------------")
    for place, playerIdAndFinishTime in ipairs(finishTimes) do
        local playerId = playerIdAndFinishTime.playerId
        print("WinCondition: Setting player "..playerId.." place to "..place)
        
        self.set_player_place {
            player_id = playerId,
            current_place = place
        }
    end

    -- Put the rest of the players at the end of the race.
    -- This is just so rest of player have some place set. All players should report their finish time.
    local nextOpenPlace = #finishTimes + 1
    getPlayersWhoDidntReportTime():each(function(playerId)
        print("WinCondition: Setting player "..playerId.." place to "..nextOpenPlace..
              " (player did not report finish time)")

        local theHero = findHeroById(playerId)

        -- Assuming that sprite will report finish line in this iteration
        if theHero~= nil and  theHero.XPos <= finish_x then

        self.set_player_place {
            player_id = playerId,
            current_place = nextOpenPlace
        }
        nextOpenPlace = nextOpenPlace + 1

        end

    end)
    print("reportPlayersPositions():: --------------ended------------------------")
end

local function addFinishTimeRecordAndReportIt(playerId, finishTime)
    print("WinCondition: addFinishTimeRecordAndReportIt("
            ..playerId..", "..finishTime..")")
            
    local finishTimeRecord = {
        playerId    = playerId,
        finishTime  = finishTime
    }
    table.insert(finishTimes, finishTimeRecord)
    
    sortFinishTimesAscending()
    traceFinishTimes()
    reportPlayersPositions()
end

game.networkingEvents.bind(EVENT_FINISHED_RACE, function(args)
    print "WinCondition: Got EVENT_FINISHED_RACE"
    addFinishTimeRecordAndReportIt(
            tonumber(args.playerId),
            tonumber(args.finishTime))
end)

-- Should be called on single player games
function g_reportLocalPlayerFinishedNow(id, finishTime)
    print("WinCondition: local player "..id.." finished race")
    addFinishTimeRecordAndReportIt(id, finishTime)
end

-- Called when hero finishes the race
function g_iFinishedRace()
    local finishTime = NETWORK_TIME
    print "WinCondition: I finished race"
    addFinishTimeRecordAndReportIt(multi_player_id_me, finishTime)
    game.networkingEvents.send(EVENT_FINISHED_RACE, {
        playerId    = tostring(multi_player_id_me),
        finishTime  = tostring(finishTime)
    })
end

-- Called once finish_x is known from all bots
function g_setBotFinishTime(botSprite)
    -- Formula is derived from the expression at hero.move_to_expected_x
    local finishTime = (1000 / botSprite.bot_speed) * finish_x + race_start_time
    print("WinCondition: bot #"..botSprite.my_id..
          " will finish at "..finishTime)
    addFinishTimeRecordAndReportIt(botSprite.my_id, finishTime)
end
    
-----------------------------------------------------------------------------------
-- Utilities
-----------------------------------------------------------------------------------

SyncedFields = {
    NOT_SYNCED_BY_DEFAULT =
        game.enums.State.VISIBLE + 
        game.enums.State.SCALE_X +
        game.enums.State.SCALE_Y +
        game.enums.State.SCALE_VELOCITY_X +
        game.enums.State.SCALE_VELOCITY_Y + 
        game.enums.State.SCALE_ACCELERATION_X + 
        game.enums.State.SCALE_ACCELERATION_Y + 
        game.enums.State.ANGULAR_VELOCITY + 
        game.enums.State.ANGULAR_ACCELERATION +
        -- Can be later enabled:
        game.enums.State.TIMERS + 
        game.enums.State.NUMERIC_PROPS + 
        game.enums.State.INSTANCE_PROPS + 
        game.enums.State.ANIMATION_ID + 
        game.enums.State.ROTATION + 
        game.enums.State.TINT,
        
    PROPERTIES = 
        game.enums.State.NUMERIC_PROPS + 
        game.enums.State.INSTANCE_PROPS,
        
    VISUALS = 
        game.enums.State.ANIMATION_ID + 
        game.enums.State.ROTATION + 
        game.enums.State.TINT
}
        

