local game = require 'game'
local sprite = game.sprite
local social = game.socialHelper
local self = sprite:create(game.gm.getLocalIndicator(0, "self"),'game_master')
local analytics = game.analyticsHelper
sprite:clearcache()


EVENT_HIT_MSG               = 18
EVENT_TALK_MSG              = 19
g_players_positions = {} -- used for power-ups
--print("g_players inited")



-----------------------------------------------------------------------------------
-- Singleplayer end game logic
-----------------------------------------------------------------------------------

local finishTimes = {} -- List of {playerId, finishTime}

local function findHeroById(id)
    local result

    forEachSprite(hero, function(currentHero)
        if currentHero.my_id == id then
            result = hero
        end
    end)

    return result
end

local function traceFinishTimes()
    for place, playerIdAndFinishTime in ipairs(finishTimes) do
        print("WinCondition: At #"..place..": "..
              playerIdAndFinishTime.playerId.." finished at "..
              playerIdAndFinishTime.finishTime)
    end
end

local function sortFinishTimesAscending()
    -- Sort by finishTime first, then by playerId
    -- In the very rare case that two players finish at the same time, the one
    -- with lower player ID wins.
    table.sort(finishTimes, function(a, b)
        if a.finishTime ~= b.finishTime then
            return a.finishTime < b.finishTime
        else
            return a.playerId < b.playerId
        end
    end)
end

local function getPlayersWhoDidntReportTime()
    local playersWhoReported = Set(map(finishTimes, function(record) return record.playerId end))
    local allPlayers = Set { 1, 2, 3, 4 }

    return allPlayers - playersWhoReported
end

local function reportPlayersPositions()

    if finish_x <= 0 then
		-- Finish line is unknown, don't report places
		return
	end
    print("reportPlayersPositions():: ========================= start ======================")
    for place, playerIdAndFinishTime in ipairs(finishTimes) do
        local playerId = playerIdAndFinishTime.playerId
        print("WinCondition: Setting player "..playerId.." place to "..place)
        
        self.set_player_place {
            player_id = playerId,
            current_place = place
        }
    end

    -- Put the rest of the players at the end of the race.
    -- This is just so rest of player have some place set. All players should report their finish time.
    local nextOpenPlace = #finishTimes + 1
    getPlayersWhoDidntReportTime():each(function(playerId)
        print("WinCondition: Setting player "..playerId.." place to "..nextOpenPlace..
              " (player did not report finish time)")

        local theHero = findHeroById(playerId)

        -- Assuming that sprite will report finish line in this iteration
        if theHero.XPos <= finish_x then

            self.set_player_place {
                player_id = playerId,
                current_place = nextOpenPlace
            }
            nextOpenPlace = nextOpenPlace + 1

        end

    end)
    print("reportPlayersPositions():: ========================= end ======================")

end

local function addFinishTimeRecordAndReportIt(playerId, finishTime)
    print("WinCondition: addFinishTimeRecordAndReportIt("
            ..playerId..", "..finishTime..")")
            
    local finishTimeRecord = {
        playerId    = playerId,
        finishTime  = finishTime
    }
    table.insert(finishTimes, finishTimeRecord)
    
    sortFinishTimesAscending()
    traceFinishTimes()
    reportPlayersPositions()
end

-- Should be called on single player games
function g_reportLocalPlayerFinishedNow(id, finishTime)
    print("g_reportLocalPlayerFinishedNow():: WinCondition: local player "..id.." finished race, time = " .. finishTime)
    addFinishTimeRecordAndReportIt(id, finishTime)
end

