local game = require 'game'
local sprite = game.sprite
local social = game.socialHelper
local self = sprite:create(game.gm.getLocalIndicator(0, "self"),'analytics_master')
local analytics = game.analyticsHelper
sprite:clearcache()


GameAnalytics = {}

--[[ Social Analytics ]]--

function GameAnalytics._getTimeFromGameStartInMinutes()
	return ELAPSED_TIME_HOURS*60 + ELAPSED_TIME_MIN_FROM_LAST_HOUR
end 

--[[
	Common fields to all analytics reports

	@return string that you can append to an analytics page-view
]]--
function GameAnalytics._commonFields(dontUseSrc)
	local coins = RAINBOW_JUICE_COINS
	local bucks = STARDUST_BUCKS
	local timeElapsed = GameAnalytics._getTimeFromGameStartInMinutes()

	local srcField = ""
	if dontUseSrc then 
		srcField = ""
	else
		srcField = "/Src" .. analytics_source
	end

	local isConnected = 0

	if (IS_FACEBOOK_LOGGED_IN == 1) then
		isConnected = 1
	elseif (IS_GOOGLE_PLUS_LOGGED_IN == 1) then
		isConnected = 1
	end

	return srcField ..
		   "/Coins" .. coins ..
   		   "/Bucks" .. bucks .. 
   		   "/IsConnected" .. isConnected ..
		   "/CurrentXP" .. current_xp ..  
		   "/LevelReached" .. current_level .. 
		   "/Launches" .. total_launches_counter ..
		   "/Sessions" .. total_sessions_counter ..
		   "/TimeFromActivation" .. timeElapsed .. 
		   "/EnergySegment" .. energy_segment_group ..
		   "/Wins" .. total_multiplayer_wins ..								
		   "/Losses"  .. (total_multiplayer_games - total_multiplayer_wins) .. 
		   "/Era" .. current_world ..
		   "/NumberOfFriends" .. total_friends_for_analytics ..
		   "/IsPaying" .. succesfull_purchases_counter ..
		   "/NumOfPets" .. (pet_total_available)..
		   "/LocalPlayerGlickoMu" .. my_rank_mu .. 
		   "/LocalPlayerGlickoPhi" .. my_rank_phi .. 
		   "/IsTutorial" .. analytics_in_tutorial
end 


--[[
(Note that Google+ is not implemented in first release)"

]]--
function GameAnalytics.reportFacebookConnectDialogOpened()
	local dontReportSrc = nil -- report dummy source
	analytics_source = 0
	local commonFields = GameAnalytics._commonFields(dontReportSrc)

	analytics.analyticsPageView("SocialConnectDialog/Open" .. 
								commonFields ..
								"/SocialDialogType" .. analytics_social_dialog_src)
end

--[[
	@param table args
		 "SocialDialogType"
		 "outcome"
				Close = 0
				Connect successful = 1
				Tried to connect and failed = 2
				Pressed connect but then cancelled without authentication = 3
]]--
function GameAnalytics.reportFacebookConnectDialogClosed(args)
	local isFullPermissions = 0

	if (IS_FACEBOOK_LOGGED_IN == 1 and IS_AUTHORIZED_TO_PUBLISH_TO_FACEBOOK == 1) then
		isFullPermissions = 1
	end

	local suffix = "/SocialDialogTypeOutcome" .. args["Outcome"] ..
				   "/IsFullPermissions" .. isFullPermissions

	local dontReportSrc = nil -- report dummy source
	local commonFields = GameAnalytics._commonFields(dontReportSrc)

	analytics.analyticsPageView("SocialConnectDialog/Close" .. 
								commonFields ..
								"/SocialDialogType" .. analytics_social_dialog_src ..
								suffix)
end

--[[
	@param args -
		event "Open" or "Close"
		"EnergyRequestsAccept"
		"EnergyGiftsAccept"
		"DidSelectAll"
]]--
function GameAnalytics.reportMessagesDialogAnalytics(args)
	local commonFields = GameAnalytics._commonFields()
	local suffix = "";

	if args["event"] == "Close" then
		if not args then
			error("You must populate args")
		end 

		suffix = "/EnergyRequestsAccept" .. args["EnergyRequestsAccept"] .. 
				 "/EnergyGiftsAccept"    .. args["EnergyGiftsAccept"] ..
				 "/DidSelectAll"         .. args["DidSelectAll"]
	end


	analytics.analyticsPageView("Messages/" .. 
								args["event"]  ..
								commonFields ..
								"/EnergyRequests" .. inbox_master.analytics_energy_requests ..
							    "/EnergyGifts" ..    inbox_master.analytics_energy_gifts ..
							    suffix)
end

--[[
	@param table args - "NotificationID", "NotificationsSent", "NotificationsUsed"
]]--
function GameAnalytics.reportPlayersLaunchedGameBecauseOfANotification(args)
	if not args then
		error("Args are missing")
	end

	local commonFields = GameAnalytics._commonFields()

	analytics.analyticsPageView("Notifications/NotificationLaunch/NotificationID" .. args["NotificationID"] ..
								"/NotificationsSent" .. args["NotificationsSent"] ..
								"/NotificationUsed" .. args["NotificationsUsed"] ..
								commonFields)
end

function GameAnalytics._getCommonPetFields()
	local petIdToEnergy = {0, 0, 0, 0, 0, 0}

	-- For each pet get it's energy
	for i = 1, limbo_pet.Length do
		local petId = limbo_pet[i].slot_id
		petIdToEnergy[petId] =  limbo_pet[i].energy_current_lives
	end

	local commonPetFields =
		"/PetOneType"        .. pet_house_slot_1 ..
		"/PetTwoType"		 .. pet_house_slot_2 .. 
		"/PetThreeType"      .. pet_house_slot_3 .. 
		"/PetFourType"       .. pet_house_slot_4 ..
		"/PetFiveType"       .. pet_house_slot_5 ..
		"/PetSixType"        .. pet_house_slot_6 ..

		"/PetOneEnergy"      .. petIdToEnergy[1] ..
		"/PetTwoEnergy"      .. petIdToEnergy[2] ..
		"/PetThreeEnergy"    .. petIdToEnergy[3] ..
		"/PetFourEnergy"     .. petIdToEnergy[4] ..
		"/PetFiveEnergy"     .. petIdToEnergy[5] ..
		"/PetSixEnergy"      .. petIdToEnergy[6] .. 
		"/NumberOfEggs"		 .. total_eggs

	return commonPetFields
end

--[[

	@param args    "event" - "Open", "Close"
]]--
function GameAnalytics.petOptionsDialog(args)
	local commonFields = GameAnalytics._commonFields()
	local commonPetFields = GameAnalytics._getCommonPetFields()	
	local petSlotFields = GameAnalytics._makePetSlotFields(args)

	local event = ""
	local closeEventFields = ""

	if args["event"] then
		event = "/" .. args["event"]
		if args["event"] == "Close" then
			closeEventFields = "/PetDialogOutcome" .. analytics_pet_dialog_outcome ..
							   "/IfPurchaseEnergy" .. analytics_increase_energy_ammount
		elseif args["event"] == "BuyLives" then
			event = event .. args["BuyLives"]
		end
	end

	

	local currPetFields =	"/SelectedPetSlot" .. selected_pet_slot_ID .. 
							"/SelectedPetType" .. curr_pet_skin .. 
							"/SelectedPetEnergy" .. selected_pet_current_energy
	
	analytics.analyticsPageView("PetOptionsDialog"   .. event .. 
								"/PetDialogType"   ..  current_screen ..								
								closeEventFields  .. 
								commonFields .. 
								currPetFields ..
								petSlotFields ..
								commonPetFields )
end

--[[

	@param
	         "event" - "Start", "End", "Quit"
]]--
function GameAnalytics.reportMatch(args)
	print("function GameAnalytics.reportMatch(args) called")
	local dontUseSrc = 1
	local commonFields = GameAnalytics._commonFields(dontUseSrc)
	local commonPetFields = GameAnalytics._getCommonPetFields()	

	-- PSAUX report
	if (args["event"] == "Start") then 
		analytics.startLevel()
	end

	local event = "/" .. args["event"]

	local matchEndFields = ""


	if (args["event"] == "End") then
		local winner = 1
		local place = 1
		local powerUpSelected = 0
		local isWinnerBot = 0

		for i=1, hero.Length do
			if hero[i].current_place == 1 then
				winner = hero[i].my_id				
				isWinnerBot = hero[i].is_shadow
			end
			if hero[i].my_id == multi_player_id_me then 
				place = hero[i].current_place
			end

			if hero[i].is_shadow == 0 then
				powerUpSelected = hero[i].applied_powerup
			end 
		end 

		matchEndFields = 		"/Winner"                 .. winner ..
								"/IsWinnerBot"			  .. isWinnerBot ..
								"/PlayerPlace"            .. place .. 
								"/PowerUpSelected"        .. curr_pet_skin  ..
								"/PowerUpUsed"            .. analytics_powerups_used ..
								"/RainbowJuiceCollected"  .. collectible_rainbow_in_session .. 
								"/RegularAcorns"          .. collectible_acorn_in_session ..
								"/SilverAcorns"           .. collectible_silver_acorn_in_session ..
								"/GoldAcorns"             .. collectible_gold_acorn_in_session ..
								"/Eggs"                   .. collectible_nest_in_session .. 
								"/SceneWithMostHits"      .. analytics_scene_with_most_hits
	end
	
	local isPlayerBot = {0,0,0}	
	if player2_rank_mu == -1 then isPlayerBot[1] = 1 end
	if player3_rank_mu == -1 then isPlayerBot[2] = 1 end
	if player4_rank_mu == -1 then isPlayerBot[3] = 1 end
	local totalBots =  isPlayerBot[1] + isPlayerBot[2] + isPlayerBot[3]

	local botsFields = 	"/IsPlayerTwoBot" .. isPlayerBot[1] ..
					  	"/IsPlayerThreeBot" .. isPlayerBot[2] .. 
					  	"/IsPlayerFourBot" .. isPlayerBot[3] .. 
						"/NumberOfBots" .. totalBots


	local playerChoiceFields = 	"/SelectedPetSlot" 		.. selected_pet_slot_ID ..
								"/SelectedPetType"		.. curr_pet_skin .. 
								"/SelectedPetEnergy"	.. selected_pet_current_energy ..
								"/HeroSetHead"			.. prop_on_head ..
								"/HeroSetMisc"			.. prop_on_torso 

	--[[local petsFields =          "/PetOneType" 			.. 
								"/PetTwoType"			..
								"/PetThreeType"			..
								"/PetFourType"			..
								"/PetFiveType"			..
								"/PetSixType"			..
								"/PetOneEnergy"			..
								"/PetTwoEnergy"     	..
								"/PetThreeEnergy"		..
								"/PetFourEnergy"		..
								"/PetFiveEnergy"		..
								"/PetSixEnergy"			..
								--]]

	analytics.analyticsPageView("Match"         .. event .. 
								commonFields .. 
								botsFields ..
								playerChoiceFields ..
								commonPetFields ..																
								matchEndFields) 
	-- PSAUX report
	if (args["event"] == "End" or args["event"] == "Quit") then 
		analytics.endLevel()
	end
	--local inLevel = analytics.isInLevelSession() -- placed here as reference
end

--[[ Other events ]] --

--[[
Events:

- Player accelerates an item
- Player pressed the accelerate button but did not actually  accelerate.


args - "event" - one of: "Accelerated", "AccelerationCancel" 
	"RushPrice"
	"TimeLeftDays"
	"TimeLeftHours"
	"TimeLeftMinutes"

]]--
function GameAnalytics.reportPlayerAccelerates(args)
	local commonFields = GameAnalytics._commonFields()
	local commonPetFields = GameAnalytics._getCommonPetFields()	


	local timeLeft = tostring(args["TimeLeftDays"]) ..
				 "d:" .. tostring(args["TimeLeftHours"]) ..
				 "h:" .. tostring(args["TimeLeftMinutes"]) .. "m"
	local totalTime = tostring(args["TotalTime"])
	local item = tostring(args["Item"])
	
	-- egg_rush_dlg (all rushes...)
	analytics.analyticsPageView("Playground/"  .. args["event"] .. 
							"/Item" .. item ..
							"/TimeLeft" .. timeLeft ..
							"/TotalTime" .. totalTime ..
							"/Price" .. args["RushPrice"] ..
							commonFields .. 
							commonPetFields)
end

--[[
1. Player upgrades an item
2. Player pressed the upgrade button but did not actually  upgrade.

@param table args
	"event" - one of: "Upgrade", "UpgradeCancel"
	"item"
]]--
function GameAnalytics.reportPlayerUpgrades(args)
	local commonFields = GameAnalytics._commonFields()
	local commonPetFields = GameAnalytics._getCommonPetFields()	

	analytics.analyticsPageView("Playground/"  ..  args["event"] .. 
							"/Item" .. args["Item"] ..
							commonFields    ..
							commonPetFields)
end

function GameAnalytics._makePetSlotFields(args)
		if(args["PetEnergy"]) then 
			return "/SelectedPetSlot" .. args["PetSlot"] ..
					"/SelectedPetType" .. args["PetType"] ..
					"/SelectedPetEnergy" .. args["PetEnergy"]
			else
			return ""
		end
end 

--[[
1. Player heals a pet
2. Player pressed the Heal button and then cancelled

@param table args
	"event" - one of: "Healed", "HealCancel", "Dismissed", "DismissCancel"
	"PetSlot"
	"PetType"
	"PetEnergy"
]]--
function GameAnalytics.reportPet(args)
	local commonFields = GameAnalytics._commonFields()
	local petSlotFields = GameAnalytics._makePetSlotFields(args)
	--if args["event"] == "DismissCancel" or args["event"]== "HealCancel" then petSlotFields = "" end
	local commonPetFields = GameAnalytics._getCommonPetFields()	

	analytics.analyticsPageView("Playground/"  .. args["event"] .. 
							commonFields    .. 
							petSlotFields   ..							
							commonPetFields)
end


--[[
1. Player completed a mission
2. Player collected a mission reward

@param table args
		
		"event" - one of "MissionCollected", "Mission", "MissionSkipped", "MissionSkipCancel"
		"MissionID" 
		"PriceAmount" - relevant only for MissionSkipped and MissionSkipCancel
]]--
function GameAnalytics.reportPlayerMission(args)

	local commonFields = GameAnalytics._commonFields()
	local commonPetFields = GameAnalytics._getCommonPetFields()	

	local priceField = ""
	if args["PriceAmount"] then
		priceField = "/Price" .. args["PriceAmount"]
	end

	analytics.analyticsPageView("Playground/"  .. args["event"] .. 
							"MissionID" .. args["MissionID"] ..
							priceField ..
							commonFields    .. 
							commonPetFields)
end


--[[
1. Player completed an Era and arrived at a gate
2. Player passed a gate and started a new era


@param table args
		
		"event" - one of "EraComplete", "EraStarted"
			"EraComplete" args:
				"GateReached"
			"EraStarted" args:
				"GatePassed"
				"GateKeyUsed"
				"PriceAmount"
				"TimeLeftDays"
				"TimeLeftHours"
				"TimeLeftMinutes"
]]--
function GameAnalytics.reportEra(args)
	local commonFields = GameAnalytics._commonFields()
	local commonPetFields = GameAnalytics._getCommonPetFields()	

	local gateFields = args["event"] .. args["GateReached"]

	if (args["event"] == "GatePassed") then

		local timeLeft = tostring(args["TimeLeftDays"]) ..
			 			"d:" .. tostring(args["TimeLeftHours"]) ..
			 			"h:" .. tostring(args["TimeLeftMinutes"]) .. "m"
				 
		gateFields = gateFields .. 
					"/GateKeyUsed" .. args["GateKeyUsed"] .. --  map_gate_dlg (result)  1 paid, 2 social - login/ask friends , INF/0 -wait
					"/Price" .. args["PriceAmount"] .. -- map_gate_dlg (self.price)
					"/TimeLeft" .. timeLeft -- map_gate_dlg.my_energy_master inst prop
	end 

	analytics.analyticsPageView("Playground/" .. 
							gateFields ..
							commonFields    .. 
							commonPetFields)
end 



--[[
1. Player reached a new level
@param table args
		"LevelUp"
		
]]--
function GameAnalytics.reportLevelUp(args)
	local commonFields = GameAnalytics._commonFields()
	local commonPetFields = GameAnalytics._getCommonPetFields()	

	-- xp_bar_lvl_complete.update_progress_bar if (tmp >= 100)
	analytics.analyticsPageView("LevelUp" .. args["LevelUp"] ..
							commonFields    .. 
							commonPetFields)
end


--[[
1. Player Gets Coins
2. Player Spends Coins - Not in first step
3. Player Gets Bucks
4  Player Spends Bucks

@param table args
	"event" - "Deposit", "Charge"
	"currencyType" - "Bucks", "Coins"
	"amount"

]]--
function GameAnalytics.reportWallet(args)
	local coins = RAINBOW_JUICE_COINS
	local bucks = STARDUST_BUCKS

	local amountField = ""
	if args["currencyType"] == "Bucks" then
		amountField = "/BucksChange" .. args["amount"]
	else
		amountField = "/" .. args["amount"]
	end
	-- wallet
	analytics.analyticsPageView("Wallet/" .. args["event"] ..
								amountField ..
								"/Coins" .. coins  ..
								"/Bucks" .. bucks ..
								"/Source" .. analytics_source)
end


--[[ Shop Events ]]--

--[[
"Player Enters Shop

Every time the shop opens. "

@param table args
	"PriceSeg"
]]--
function GameAnalytics.reportPlayerEnterShop(args)
	local commonFields = GameAnalytics._commonFields()
	local commonPetFields = GameAnalytics._getCommonPetFields()	

	analytics.analyticsPageView("Price" .. args["PriceSeg"] ..
							"/Store/Enter" ..
							commonFields    .. 
							commonPetFields)
end

--[[
1. Player buys bucks/coins after not having enough.
2. Player doesn't have enough coins/bucks but won't buy more.
3. Buy Currency not through "not enough"

@param table args
	"What" - "Coins" or "Bucks"
	"Item" 
	"PurchaseState" - 1 -Success, 2- Failed, 3 Canceled
	"Pack" - This is the ID number of the coins/buck pack
	"Cancel" - if true, reports event (2)
	"NotThroughNotEnough" - if true reports event (3)
]]--
function GameAnalytics.reportPlayerNotEnough(args)
	local commonFields = GameAnalytics._commonFields()
	local commonPetFields = GameAnalytics._getCommonPetFields()	

	local eventFields = ""
	if args["Cancel"] then
		eventFields = "/Cancel"
	else
		local purchase_state = tonumber(args["PurchaseState"])
		if(purchase_state == 3) then 
			purchase_state = 4
		elseif(purchase_state == 4) then 
			purchase_state = 3
		end


		if args["NotThroughNotEnough"] == nil then eventFields = "/Item" .. args["Item"] end


		eventFields = eventFields .. "/GetMoreBucks" .. 
					  "/Purchase" .. tostring(purchase_state) ..
					  "/" .. args["What"] .. "Pack" .. args["Pack"]
	end

	local notEnoughField = "/NotEnough" .. args["What"];
	if args["NotThroughNotEnough"] then
		notEnoughField = ""
	end 

	analytics.analyticsPageView("Price" .. args["PriceSeg"] ..
							"/Store" .. notEnoughField ..
							eventFields ..							
							commonFields    .. 
							commonPetFields)
end

--[[
1. Immediate Close

@param table args
	"PriceSeg"
]]--
function GameAnalytics.reportImmediateClose(args)
	local commonFields = GameAnalytics._commonFields()
	local commonPetFields = GameAnalytics._getCommonPetFields()	

	analytics.analyticsPageView("Price" .. args["PriceSeg"] ..
							"/Store/CloseAtOnce" ..
							commonFields    .. 
							commonPetFields)
end

--[[
1. Purchase of items (Not currency)

@param table args
	"PriceSeg"
	"Item"
]]--
function GameAnalytics.reportPurchaseItem(args)
	local commonFields = GameAnalytics._commonFields()
	local commonPetFields = GameAnalytics._getCommonPetFields()	

	analytics.analyticsPageView("Price" .. args["PriceSeg"] ..
							"/Store" ..
							"/Item" .. args["Item"] ..
							"/UnlockSuccessful" ..
							commonFields    .. 
							commonPetFields)
end

--[[
1. Enter Currency Category

@param table args
	"PriceSeg"
	"Item"
]]--
function GameAnalytics.reportEnterCurrencyCategory(args)
	local commonFields = GameAnalytics._commonFields()
	local commonPetFields = GameAnalytics._getCommonPetFields()	

	analytics.analyticsPageView("Price" .. args["PriceSeg"] ..
							"/Store" ..
							"/GetMoreBucks/Enter" ..
							commonFields    .. 
							commonPetFields)
end

--[[
1. Player pressed the set link button in the store.

@param table args
	"PriceSeg"
	"ItemCurrent"
	"ItemShown"
]]--
function GameAnalytics.reportPlayerPressedSetLinkButton(args)
	local commonFields = GameAnalytics._commonFields()
	local commonPetFields = GameAnalytics._getCommonPetFields()	

	analytics.analyticsPageView("Price" .. args["PriceSeg"] ..
							"/Store/SetLinkButton/ItemShown" .. args["ItemShown"] ..
							"/ItemCurrent" .. args["ItemCurrent"] ..
							commonFields    .. 
							commonPetFields)
end

--[[
1. Player rushed and item in the store (pressed the "Rush" button in the rush dialog)
2. Player chose not to rush an item (pressed to buy item, saw the rush dialog, and cancelled)

@param table args
	"event" - "Rushed" or "RushCancelled"
	"PriceSeg"
	"RushedItem"
	"PurchasedItem"
	"TimeLeft"
	"TotalTime"
	"PriceAmount"
]]--
function GameAnalytics.reportPlayerRush(args)
	local commonFields = GameAnalytics._commonFields()
	local commonPetFields = GameAnalytics._getCommonPetFields()	

	analytics.analyticsPageView("Price" .. args["PriceSeg"] ..
							"/Store/".. args["event"] ..  "/RushedItem" .. args["RushedItem"] ..
							"/PurchasedItem" .. args["PurchasedItem"] ..
							"/TimeLeft" .. args["TimeLeft"] .. 
							"/TotalTime" .. args["TotalTime"] .. 
							"/Price" .. args["PriceAmount"] ..
							commonFields    .. 
							commonPetFields)
	
end

