local game = require 'game'
local sprite = game.sprite
local social = game.socialHelper
local self = sprite:create(game.gm.getLocalIndicator(0, "self"),'rank_master')
local analytics = game.analyticsHelper
sprite:clearcache()


local RATING_INFO_EVENT = 0x110 -- Args: playerNumber, mu, phi
local TAU = 0.3 -- See Glicko2 paper about tau value.

local g2 = Glicko2(1500, TAU)

local playersRatings = {} -- {playerNumber: rating}

local glickoRecievedReports = 0

-- Local

local function tracePlayerRatings()
    for playerNumber, rating in pairs(playersRatings) do
        print("rank_master: Rating #"..playerNumber..": "..rating:toString())
    end
end

local function forEachHeroOtherThanMine(func)
    for i = 1, hero.Length do
        if hero[i].my_id ~= multi_player_id_me then
            func(hero[i])
        end
    end
end

game.networkingEvents.bind(RATING_INFO_EVENT, function(args)
    local playerNumber = tonumber(args.playerNumber)
    local rating = Rating(tonumber(args.mu), tonumber(args.phi))
    storeOtherPlayerRankInGlobals(tonumber(args.mu), tonumber(args.phi))
    glickoRecievedReports = glickoRecievedReports + 1

    print("rank_master: Got rating for #"..playerNumber..": "..
          rating:toString())
          
    playersRatings[playerNumber] = rating
    
    tracePlayerRatings()

	--if(glickoRecievedReports == multi_players_cnt-1) then         
    --   analytics_master.match_start({})
    --end
end)

function  storeOtherPlayerRankInGlobals(mu, phi)
    if(player2_rank_mu == -1) then 
        player2_rank_mu = mu
        player2_rank_phi = phi
    elseif(player3_rank_mu == -1) then 
        player3_rank_mu = mu
        player3_rank_phi = phi
    elseif(player4_rank_mu == -1) then 
        player4_rank_mu = mu
        player4_rank_phi = phi
    end
end

local function getMyPlace()
    print("hero.Length="..tostring(hero.Length))
    for i = 1, hero.Length do
        if hero[i].my_id == multi_player_id_me then
            return hero[i].current_place
        end
    end
    
    assert(false)
end

local function getMyRating()
    return Rating(my_rank_mu, my_rank_phi, my_rank_sigma)
end

local function setMyRating(newRating)
    my_rank_mu = newRating.mu
    my_rank_phi = newRating.phi
    my_rank_sigma = newRating.sigma
end

local function isLosingTooMuch()
    if total_multiplayer_games == 0 then
        return false
    end
    
    local winPercentage = total_multiplayer_wins * 100 / total_multiplayer_games
    
    print("rank_master: win precentage = "..winPercentage.."%")
    
    return winPercentage < MIN_WIN_PERCENTAGE
end

local function getLevel(div, mu, phi)
    if phi * 4 < div then
        return math.floor(mu / div)
    else
        return -1
    end
end

local function calculateLevels()
    local mu = my_rank_mu
    local phi = my_rank_phi

    if isLosingTooMuch() then
        print("rank_master: Win percentage too low!")
        mu = mu / 2
        phi = phi * 2 / 3
    end
    
    my_rank_l1 = getLevel(RANK_DIV_L1, mu, phi)
    my_rank_l2 = getLevel(RANK_DIV_L2, mu, phi)
    my_rank_l3 = getLevel(RANK_DIV_L3, mu, phi)
end

-- Global

--[[
    Shares rating with other players, called at the beginning of each race.
]]
function g_sendMyRatingInfo()
    game.networkingEvents.send(RATING_INFO_EVENT, {
        playerNumber = tostring(multi_player_id_me),
        mu = tostring(my_rank_mu),
        phi = tostring(my_rank_phi),
    })
end

--[[
    Updates my_rank_mu, my_rank_phi and my_rank_sigma with the results of the 
    last race. Called at the end of each race.
    Recalculates my_rank_l1, my_rank_l2 and my_rank_l3 accordingly.
]]
function g_calculateNewRating()
    print("rank_master: Calculating new rank")
    tracePlayerRatings()

    local gameResults = {}
    local myPlace = getMyPlace()
    
    total_multiplayer_games = total_multiplayer_games + 1
    if myPlace == 1 then
        total_multiplayer_wins = total_multiplayer_wins + 1
    end
    
    print("rank_master: total_multiplayer_games = "..total_multiplayer_games)
    print("rank_master: total_multiplayer_wins = "..total_multiplayer_wins)
    
    forEachHeroOtherThanMine(function(heroSprite)
        local result
        
        if playersRatings[heroSprite.my_id] == nil then
            print("rank_master: #"..heroSprite.my_id.." didn't report rating")
            return
        end
        
        if (heroSprite.current_place == 0) or
                (heroSprite.current_place > myPlace) then
            print("rank_master: WIN against #"..heroSprite.my_id)
            result = WIN
        else
            print("rank_master: LOSS against #"..heroSprite.my_id)
            result = LOSS
        end
        
        table.insert(gameResults, {result, playersRatings[heroSprite.my_id]})
    end)
    
    if #gameResults == 0 then
        print("rank_master: Didn't get any rating of other players")
        return
    end
    
    local myOldRating = getMyRating()
    print("rank_master: My old rating: "..myOldRating:toString())
    local myNewRating = g2:rate(myOldRating, gameResults)
    print("rank_master: My new rating: "..myNewRating:toString())
    setMyRating(myNewRating)
    calculateLevels()
end

--[[
    Gets the roomProperties table (to be passed to startRandomGame or
    startGameWithFriends) and adds properties so players will be matched with
    players of the same skill.
]]
function g_addPropertiesForMyLevel(roomProperties)
    if debug_use_ranks == 0 then --or qa_ignore_glicko == 1 then
        return
    end
        
    if total_sessions_counter == 0 then 
	   roomProperties["world"] = 0
    else
       roomProperties["world"] = current_world
    end

        -- new "glicko" 1250/1750 (Tzach's algo) by my_rank_mu
    local qMu 
    if my_rank_mu < 1250 then 
        qMu = 1
    elseif my_rank_mu < 1750 then 
        qMu = 2
    else
        qMu = 3
    end
	
	if qa_ignore_glicko == 1 then
		qMu = 0
        print("glicko is ignore")
	end	
    roomProperties["qMu"] = qMu    

    print("rank_master: properties: world = " .. roomProperties["world"] .. 
                                    "qMu = " .. roomProperties["qMu"] .. 
                                    "realMu = " .. my_rank_mu )                                
end

-- Main

calculateLevels()
