local game = require 'game'
local sprite = game.sprite
local social = game.socialHelper
local self = sprite:create(game.gm.getLocalIndicator(0, "self"),'pets_storage_master')
local analytics = game.analyticsHelper
sprite:clearcache()


PetsDataStorage = {}
print("--------- PetsDataStorage initialized")
PetsDataStorage.platform 	= require 'platform'
PetsDataStorage.filepath 	= PetsDataStorage.platform.getSandboxStoragePath() .. "/pets_data_storage.txt"
PetsDataStorage.file 		= nil
PetsDataStorage.slots       = {}
PetsDataStorage.totalSlots	= 6
----------------------------------------------------------------------------------------------------------
--arg "w"/"a"/"r"
function PetsDataStorage:OpenFile(arg)	
	self.file = io.open(PetsDataStorage.filepath, arg)
	return self.file
end
-----------------------------------------------------------------------
function PetsDataStorage:CloseFile()
	if(self.file ~= nil) then 
		self.file:close() 
		self.file = nil 
	end
end
----------------------------------------------------------------------------------------------------------
function PetsDataStorage:Init()
	self:OpenFile("r")
	if self.file then 
		-- read pets data from file and insert to the slots table
		for i=1, self.totalSlots do
			self.slots[i] = CPetData:ReadFromFile(self.file)
		end
		print("PetsDataStorage::" .. #self.slots .. " slots loaded from file")
		self:CloseFile()
	else
		print("PetsDataStorage:: first run -> creating {0,0,0...} slots")
		for i=1, self.totalSlots do
			self.slots[i] = CPetData()
		end
	end
	self:PrintSlots("slots after init")	
end
----------------------------------------------------------------------------------------------------------
function  PetsDataStorage:SaveSlotsToFile()
	self:OpenFile("w")
	if self.file then 
		for i=1,self.totalSlots do
			self.slots[i]:SaveToFile(self.file)
		end
		self:CloseFile()
		print("PetsDataStorage::" .. #self.slots .. " slots saved to file")
	else
		print("This should never happen:: PetsDataStorage:SaveSlotsToFile() get file = nil")
	end
end
----------------------------------------------------------------------------------------------------------
function PetsDataStorage:SavePetDataToSlot(slotNum, petType, pwr1, pwr2, pwr3, rarity, cooldown)
	local data = CPetData(petType, pwr1, pwr2, pwr3, rarity, cooldown)	
	print("PetsDataStorage:SavePetDataToSlot[" .. slotNum .. "] = " .. data:ToString())	
	self.slots[slotNum] = data
end
----------------------------------------------------------------------------------------------------------
function  PetsDataStorage:GetPetDataFromSlot(slotNum)
	local data = self.slots[slotNum]
	print("PetsDataStorage:GetPetDataFromSlot[" .. slotNum .. "] = " .. data:ToString())	
	return data
end
----------------------------------------------------------------------------------------------------------
function PetsDataStorage:RemovePet(slotNum)
	-- find last slot if there is one
	local lastSlotIndex = 0
	for i=self.totalSlots,1,-1 do 
		if self.slots[i].petType ~= 0 then
			lastSlotIndex = i
			break
		end
	end
	print("PetsDataStorage:RemovePet:: lastSlotIndex = " .. lastSlotIndex .. ", slotNum = " .. slotNum)
	self:PrintSlots(" before removing slot[" .. slotNum .. "] ")
	if lastSlotIndex > slotNum then
		self.slots[slotNum] = self.slots[lastSlotIndex]	
		self.slots[lastSlotIndex] = CPetData()
	else
		print("PetsDataStorage:RemovePet:: Last slot do not exist!")
		self.slots[slotNum] = CPetData()	
	end	
	
	self:PrintSlots(" after removing slot[" .. slotNum .. "] ")
end
----------------------------------------------------------------------------------------------------------
-- for debug use
function  PetsDataStorage:PrintSlots(title)
	local slotsTitle = title or "pets slots data"
	local txt = "--------------- " .. slotsTitle .. "-------------------\n"
	for i=1, self.totalSlots do
		txt = txt .. "slot[" .. i .. "]=" .. self.slots[i]:ToString() .. "\n"
	end
	txt = txt .. "----------------------------------------------------"
	print(txt)
end
----------------------------------------------------------------------------------------------------------
function PetsDataStorage:GetSkinTypes()
	local skins = {}
	for i=1, self.totalSlots do
		skins[i] = self.slots[i].petType		
	end
	return skins
end
----------------------------------------------------------------------------------------------------------
PetsDataStorage:Init()

