local game = require 'game'
local sprite = game.sprite
local social = game.socialHelper
local self = sprite:create(game.gm.getLocalIndicator(0, "self"),'preplay_master')
local analytics = game.analyticsHelper
sprite:clearcache()


EVENT_GOTO_GAME_ROOM 		= 1
EVENT_PLAYER_DATA			= 2
EVENT_START_PREPLAY_TIMER   = 3
EVENT_REQUEST_PLAYER_ID     = 4
EVENT_PLAYER_ID_RESPONSE    = 5
EVENT_BOT_DATA 			    = 6

g_PreplayNames = {}
function g_reset_preplay()
    last_joined_player_name = "Anonymous"
    multi_player_my_serial_number = -1
    multi_player_id_me = 0
    startGameArgs = false
    multi_players_cnt = 0;
    g_PreplayNames = {}
end

-- reset----------------------------------------------------------------------------------
g_MySocialId = -1
startGameArgs = false
g_reset_preplay();

if total_sessions_counter == 0 then 
	rank_for_scenes = 0
else
	rank_for_scenes = current_world
end


PlayerTracker = class()

-- Static
function PlayerTracker:handleStartGameArgs(args)
    local newIdsMapping = self:_convertMessageTableToIntTable(args)
    self:_traceMapping(args)

    if (newIdsMapping[multi_player_my_serial_number] ~= nil) then
        self:_updateMyPlayerNumber(newIdsMapping)
        self:_updatePlayerFrames(newIdsMapping)   
    else
        print "!!! BUG !!! My serial number is not in newIdsMapping"
        print "!!! BUG !!! Disconnecting on purpose"
        game.networking.disconnect()
    end
end

function PlayerTracker:_traceMapping(mapping)
    print("preplay_init: handling game start args: ")
	table.foreach(mapping, function(k, v) print(k.." = "..v) end)
end

function PlayerTracker:_updateMyPlayerNumber(mapping)
    assert(mapping[multi_player_my_serial_number] ~= nil)
    
    print("preplay_init: Changing my ID from "..
          multi_player_my_serial_number.." to "..mapping[multi_player_my_serial_number])
          
    multi_player_id_me = mapping[multi_player_my_serial_number]
end

function PlayerTracker:_updatePlayerFrames(mapping)
    forEachSprite(preplay_player_frame, function(frame)        
		local oldId = frame.player_id
        local newId = mapping[oldId]
        if newId ~= nil then
            print("preplay_init: updating player frame: "..oldId.." -> "..newId)
            frame.player_id = newId
        else
            print("preplay_init: frame of player "..oldId.." not updated")
        end
    end)
end

-- Non static
function PlayerTracker:new(myPlayerNumber)
    self._players = Set()
    self._highestSerialNumber = myPlayerNumber
end

function PlayerTracker:playerJoined(playerNumber)
    self._players:add(playerNumber)
    
    print("preplay_init: player #"..playerNumber.." added")
    self._highestSerialNumber =
            math.max(self._highestSerialNumber, playerNumber)

    self:getStartGameMessageArgs() -- Just for prints
end

function PlayerTracker:playerLeft(playerNumber)
    self._players:remove(playerNumber)
    
    print("preplay_init: player #"..playerNumber.." Removed")
    self:getStartGameMessageArgs() -- Just for prints
end

function PlayerTracker:getStartGameMessageArgs()
    local args = {}
    print("preplay_init: player #"..tostring(multi_player_my_serial_number).." (me) will have id 1")
    args[tostring(multi_player_my_serial_number)] = "1"
    local nextPlayerId = 2
    
    self._players:each(function(playerNumber)
        print("preplay_init: player #"..playerNumber.." will have id "..
                nextPlayerId)
        
        args[tostring(playerNumber)] = tostring(nextPlayerId)
        nextPlayerId = nextPlayerId + 1
    end)

    local nextSerialNumber = self._highestSerialNumber + 1
    
    -- Possible fix for #10277:
    -- Pad with next serial numbers
    while nextPlayerId <= 4 do
        print("preplay_init: player #"..nextSerialNumber.." will have id "..
                nextPlayerId .. " (safety)")

        args[tostring(nextSerialNumber)] = tostring(nextPlayerId)
        nextPlayerId = nextPlayerId + 1
        nextSerialNumber = nextSerialNumber + 1
    end
    
    return args
end

function PlayerTracker:_convertMessageTableToIntTable(tbl)
    local result = {}
    for k, v in pairs(tbl) do
        result[tonumber(k)] = tonumber(v)
    end
    return result
end

g_playerTracker = PlayerTracker(1)

----Binds
------------------------------------------------------------------------------------------
game.networkingEvents.bind(game.enums.EngineEvents.JOINED_ROOM, function(args)

		if(my_fb_id > 0) then 
			g_MySocialId = game.networking.friendIdToPlayerId(my_fb_id) or -1
		else
			g_MySocialId = -1
		end
		multi_players_cnt = game.networking.getNumberOfPlayersInRoom()
        multi_player_id_me = tonumber(args.playerNumber)		
        multi_player_my_serial_number = multi_player_id_me        
        self.on_join_game_listener({player_id = multi_player_id_me })
        g_playerTracker = PlayerTracker(multi_player_id_me )        
        
		print("PREPLAY.JOINED_ROOM:: player " .. multi_player_id_me .. 
					" joined to room with ".. multi_players_cnt..", " .. 
					"soc id = " .. g_MySocialId .. ", my_fb_id = " .. my_fb_id)        
		
		if (preplay_debug_line.Length > 0) then
        	preplay_debug_line.canvas["txt"].text = "me joined id=" .. multi_player_id_me
        end
        
        if(IS_HOST == 0) then 
        	SendMyDataToOtherPlayers(-1) 
        else	
        	print("rank_for_scenes in HOST(id=".. multi_player_id_me .. ") is " .. rank_for_scenes)
    	end
        
        if startGameArgs then
            PlayerTracker:handleStartGameArgs(startGameArgs)
            startGameArgs = false
        end        
    end)
------------------------------------------------------------------------------------------
game.networkingEvents.bind(game.enums.EngineEvents.LEFT_ROOM, function(args)
		print("PREPLAY.LEFT_ROOM event")
        multi_player_my_serial_number = -1
        multi_player_id_me = 0 
        multi_players_cnt = 0
        startGameArgs = false
    end)
------------------------------------------------------------------------------------------
game.networkingEvents.bind(game.enums.EngineEvents.GAME_STARTED, function(args)
        self.on_start_game_listener({})
    end)
------------------------------------------------------------------------------------------
game.networkingEvents.bind(game.enums.EngineEvents.PLAYER_JOINED_ROOM, function (args)		
        local joinedPlayerNumber = tonumber(args.playerNumber)
		OnPlayerJoined(joinedPlayerNumber)
		preplay_master.start_timer_to_send_my_data_to({id = joinedPlayerNumber})
		--SendMyDataToOtherPlayers(joinedPlayerNumber)
				
		if (preplay_debug_line.Length > 0) then
			preplay_debug_line.canvas["txt"].text = "player id=" .. args.playerNumber .. "joined, total = " .. multi_players_cnt .. "/" .. multi_players_total
		end
		
		-- if all players have been connected --
		if(multi_players_cnt == multi_players_total) then
			print("PREPLAY:: Start count down timer, after player " .. joinedPlayerNumber .. " joined")
            g_StartCountDownAndGoToGame()
		end
	end)
------------------------------------------------------------------------------------------
game.networkingEvents.bind(game.enums.EngineEvents.CONNECTED, function (args)
        self.on_connect_success()
	end)
------------------------------------------------------------------------------------------
game.networkingEvents.bind(game.enums.EngineEvents.DISCONNECTED, function (args)
		self.on_disconnect({})
	end)
------------------------------------------------------------------------------------------
game.networkingEvents.bind(game.enums.EngineEvents.CONNECTION_FAILURE, function (args)		
		self.on_connection_failure({})
	end)
------------------------------------------------------------------------------------------
game.networkingEvents.bind(game.enums.EngineEvents.JOIN_FAILURE, function (args)
		self.on_join_failure({})		
	end)
------------------------------------------------------------------------------------------
game.networkingEvents.bind(EVENT_GOTO_GAME_ROOM, function (args)
	    if multi_player_my_serial_number >= 0 then
	        PlayerTracker:handleStartGameArgs(args)
        else
            -- JOINED_ROOM not called yet
            startGameArgs = args
        end
		self.on_goto_game_room({})
	end)
------------------------------------------------------------------------------------------
game.networkingEvents.bind(EVENT_START_PREPLAY_TIMER, function (args)
		print("FB:: player#" .. multi_player_id_me .. " received EVENT_START_PREPLAY_TIMER event")
	    if multi_player_my_serial_number >= 0 then
	        PlayerTracker:handleStartGameArgs(args)
        else
            -- JOINED_ROOM not called yet
            startGameArgs = args
        end
		self.start_goto_game_room_timer({})
	end)
------------------------------------------------------------------------------------------
game.networkingEvents.bind(game.enums.EngineEvents.PLAYER_LEFT_ROOM, function (args)
		print("PREPLAY:: player #" .. args.playerNumber .. "left room, " .. 
			"player #" .. multi_player_id_me .. "become host = " .. IS_HOST)
		OnPlayerLeft(tonumber(args.playerNumber))
	end)
---------------------------------------------------------------------------------------
game.networkingEvents.bind(EVENT_PLAYER_DATA, function (args)
		print("FB:: player#" .. multi_player_id_me .. 
				"get data from#" .. tonumber(args.from) .. 
				" addressed to#" .. tonumber(args.to))		
		if(tonumber(args.to) == -1 or tonumber(args.to) == multi_player_id_me) then
			--(player_id, pet_id, level, world, hat, misc, rank_for_scenes, robot_color, social_id)--
			OnReceivingDataFromPlayer(tonumber(args.from), tonumber(args.pet_id), tonumber(args.level), 
				tonumber(args.world), tonumber(args.hat_prop), tonumber(args.misc_prop), 
				args.name, tonumber(args.rank_for_scenes), tonumber(args.pet_robot_color), args.social_id)			
		end
	end)


 ---Helpers
 ----------------------------------------------------------------------------------------
function  SendMyDataToOtherPlayers(dst_id)
	local data = {}
	data.from = multi_player_id_me
	data.to   = dst_id		-- -1 for all
	data.level = current_level
	data.pet_id = curr_pet_skin	
	data.hat_prop = prop_on_head
	data.misc_prop = prop_on_torso
	data.world = current_world
	data.name = NamingMaster.name or "Anonymous" -- use player's name
	data.social_id = g_MySocialId or -1
	data.pet_robot_color = pet_robot_color		
	
	local myWorld = current_world
	if total_sessions_counter == 0 then myWorld = 0  end
	
	if (IS_HOST == 1) then 
		data.rank_for_scenes = myWorld
		print("rank_for_scenes sended by host(id=" .. data.from.. ") to= ".. data.to.. " is " .. data.rank_for_scenes)
	else
		data.rank_for_scenes = -1 -- it was myWorld, only host should send the rank
	end
	game.networkingEvents.send(EVENT_PLAYER_DATA, data)
	print("PREPLAY:: player " .. data.name .. " #" .. multi_player_id_me .. "soc_id = " .. g_MySocialId .. " send his data to#" .. data.to)
end
----------------------------------------------------------------------------------------
function  OnPlayerJoined(player_id)
	preplay_master.on_player_joined({player_id = player_id})
end
----------------------------------------------------------------------------------------
function  OnPlayerLeft(player_id)
	preplay_master.on_player_left({player_id = player_id})
end
----------------------------------------------------------------------------------------
function  OnReceivingDataFromPlayer(player_id, pet_id, level, world_, hat_, misc_, name_, rank_for_scenes_, pet_robot_color_, social_id_)
	g_receiving_data_from_player_name = name_
	g_receiving_data_from_player_social_id = social_id_

	preplay_master.on_receiving_data_from_player(
													{ player_id = player_id, 
													  pet_id = pet_id,
													  level = level, 
													  world_ = world_,
													  hat_ = hat_,
													  misc_ = misc_, 
													  rank_for_scenes_ = rank_for_scenes_, 
													  pet_robot_color_ = pet_robot_color_}
											    )
end
-----------------------------------------------------------------------------------------
function g_StartCountDownAndGoToGame(debugMode)
    if (IS_HOST == 1 or debugMode) then
        game.networking.setIsRoomOpen(false)
        
        local args = g_playerTracker:getStartGameMessageArgs()
        game.networkingEvents.send(EVENT_START_PREPLAY_TIMER, args)
        PlayerTracker:handleStartGameArgs(args) -- Also update locally
        print("YAN:: self.start_goto_game_room_timer({})")
        self.start_goto_game_room_timer({})
	else
		print("YAN:: Strange!, g_StartCountDownAndGoToGame() call and not host")
    end
end
------------------------------------------------------------------
--     BOTS
------------------------------------------------------------------
math.randomseed( tonumber(tostring(os.time()):reverse():sub(1,6)) )

function g_CreateBotsOnTimeOut(debugMode)	

	if (IS_HOST == 1 or debugMode == true) then
		local botsToCreate = multi_players_total - multi_players_cnt

		if debugMode then botsToCreate = 3 end
		print("BOTS ::  botsToCreate = " .. botsToCreate)
		local botsData = {}
		for i=1, botsToCreate do
			botsData[i] = g_CreateRandomBotData(multi_players_cnt + i, i)		
		end
		
		--print("PREPLAY TEST ::  before flattening data")
		--flatten the data to one table
		local args = {}
		for i=1, botsToCreate do
			for k,v in pairs(botsData[i]) do
				args[k] = v
			end
		end
		args["totalBots"] = tostring(botsToCreate)

		if debugMode then 
			g_ExtractBotsDataFromArgs(args)
		else
			print("BOTS:: game.networkingEvents.send(EVENT_BOT_DATA, args) sent")			
			g_ExtractBotsDataFromArgs(args)
			game.networkingEvents.send(EVENT_BOT_DATA, args)					
			game.networking.setIsRoomVisible(false)
		end
	else
		print("BOTS:: should never happens: inside g_CreateBotsOnTimeOut() IS_HOST = " .. IS_HOST .. "debugMode = " .. (debugMode or "false"))

	end

end
--------------------------------------------------------------------
game.networkingEvents.bind(EVENT_BOT_DATA, function(args)
			if(IS_HOST == 0) then
				g_ExtractBotsDataFromArgs(args)
				-- unbind?
			else
				print("EVENT_BOT_DATA = Should never happen!!!")
			end	
    end)

------------------------------------------------------
function g_CreateRandomBotData(botID, index)
	local data = {}
	local hat = g_GetRandomHatProp()
	local misc = g_GetRandomMiscProp()
	local pet = g_GetRandomPet()
	local world = g_GetRandomWorld()
	local name = g_GetRandomName()
	local robotSkin = 0
	if(pet == PET_ROBOT) then robotSkin = math.random(1,5) end
	
	g_DebugPrintBotData("ON CREATE:: ", botID, hat, misc, pet, world, robotSkin, name)
	
	data["hat" .. index] 	= tostring(hat)
	data["misc" .. index] 	= tostring(misc)
	data["pet" .. index] 	= tostring(pet)
	data["world" .. index] 	= tostring(world)
	data["id" .. index] 	= tostring(botID)
	data["robotSkin" .. index] 	= tostring(robotSkin)
	data["name" .. index] = name
	return data
end
------------------------------------------------------
function g_GetRandomHatProp()
	local t = math.random(8)
	local h = 0
	if 		t == 1 then h = PROP_KING_CROWN
	elseif  t == 2 then h = PROP_PIRATE_HAT
	elseif  t == 3 then h = PROP_CAVEMAN_HAT
	elseif  t == 4 then h = PROP_WIZARD_HAT
	elseif  t == 5 then h = PROP_DEMON_HELMET
	elseif  t == 6 then h = PROP_SOLDIER_HELMET
	elseif  t == 7 then h = PROP_ROBOT_HELMET
	end
	return h
end
------------------------------------------------------
function g_GetRandomMiscProp()
	local t = math.random(8)
	local h = 0
	if 		t == 1 then h = PROP_KING_CAPE
	elseif  t == 2 then h = PROP_PIRATE_BELT
	elseif  t == 3 then h = PROP_CAVEMAN_CAPE
	elseif  t == 4 then h = PROP_WIZARD_CAPE
	elseif  t == 5 then h = PROP_DEMON_CAPE
	elseif  t == 6 then h = PROP_SOLDIER_BELT
	elseif  t == 7 then h = PROP_ROBOT_BELT
	end
	return h
end
------------------------------------------------------
function g_GetRandomPet()
	local t = math.random(2,10)
	-- no pets for bots in world 0
	if total_sessions_counter == 0 then t = 0  end	
	local h = 0
	if 		t == 1 then h = PET_ROBOT
	elseif  t == 2 then h = PET_CHICK
	elseif  t == 3 then h = PET_GOLD_DRAGON
	elseif  t == 4 then h = PET_DEMON
	elseif  t == 5 then h = PET_PURPLE_DRAGON
	elseif  t == 6 then h = PET_ASTROCAT
	elseif  t == 7 then h = PET_ALIEN
	elseif  t == 8 then h = PET_ANGEL
	elseif  t == 9 then h = PET_DEATH_ANGEL
	end
	return h
end
------------------------------------------------------
function g_GetRandomWorld()
	return math.random(5)
end
------------------------------------------------------
function g_GetRandomName()
	local name = "Anonymous"
	if (g_RandomNames ~= nil) then 
		if (#g_RandomNames > 0) then
			if math.random(100) < 80 then 
				name = g_RandomNames[math.random(#g_RandomNames)]
			else				
				name = "User_" .. math.random(9) .. math.random(9) .. math.random(9) .. math.random(9)
			end
		end
	end
	return name
end
------------------------------------------------------
function g_ExtractBotsDataFromArgs(args)	
	local totalBots = tonumber(args["totalBots"])
	
	for i=1, totalBots do
		g_ProcessBotData(	tonumber(args["id" .. i]), 
							tonumber(args["hat" .. i]),
							tonumber(args["misc" .. i]),
							tonumber(args["pet" .. i]),
							tonumber(args["world" ..i]),
							tonumber(args["robotSkin" .. i]),
						 	 		 args["name" .. i]
						)
	end
end
------------------------------------------------------
function g_ProcessBotData(id, hat, misc, pet, world, robotSkin, name)
	
	g_receiving_data_from_player_name = name

	preplay_master.on_procces_bot_data({id = id, 
										hat = hat, 
										misc = misc, 
										pet = pet, 
										world = world, 
										robotSkin = robotSkin})
end
------------------------------------------------------
function g_DebugPrintBotData(prefix, id, hat, misc, pet, world, robotSkin, name)
	print((prefix or "") .. "Bot[" .. id .. "] = {hat = " .. hat .. ", misc = " .. misc .. 
							",pet = " .. pet .. ", world = " .. world .. ", robotSkin = " .. robotSkin .. ", name = ".. name .. "}" )
end
------------------------------------------------------
function g_DebugUnitTest()
	g_CreateBotsOnTimeOut(true) -- run in debug mode
end




